import factory

from technology_radar import models


class AreaFactory(factory.DjangoModelFactory):
    name = 'Tools'

    class Meta:
        model = models.Area


class StatusFactory(factory.DjangoModelFactory):
    name = 'Access'

    class Meta:
        model = models.Status


class RadarFactory(factory.DjangoModelFactory):
    name = 'Moor Interactive'

    class Meta:
        model = models.Radar


class BlipFactory(factory.DjangoModelFactory):
    area = factory.SubFactory(AreaFactory)
    radar = factory.SubFactory(RadarFactory)
    status = factory.SubFactory(StatusFactory)
    name = 'BEM'
    body = 'Block, element and modifiers.'

    class Meta:
        model = models.Blip



from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.views import APIView


__all__ = ['RootView']


class RootView(APIView):
    def get(self, request, format=None):
        return Response({
            'area-list': reverse(
                'api-area-list', request=request, format=format),
            'status-list': reverse(
                'api-status-list', request=request, format=format),
            'radar-list': reverse(
                'api-radar-list', request=request, format=format),
            'blip-list': reverse(
                'api-blip-list', request=request, format=format)
        })



from django.contrib import admin

from simple_history.admin import SimpleHistoryAdmin

from technology_radar import models


admin.site.register(models.Area)
admin.site.register(models.Status)
admin.site.register(models.Radar)
admin.site.register(models.Blip, SimpleHistoryAdmin)



from django.http import HttpResponse

from reportlab.pdfgen import canvas


class AbstractRenderer(object):
    def render(self, request, radar):
        return HttpResponse('')


class JPGRenderer(AbstractRenderer):
    pass


class PDFRenderer(AbstractRenderer):
    def render(self, request, radar):
        response = HttpResponse(content_type='application/pdf')
        response['Content-Disposition'] = 'attachment; filename="%s.pdf"' % (
                                          radar.slug)

        pdf = canvas.Canvas(response)
        pdf.drawString(100, 100, radar.name)
        pdf.showPage()
        pdf.save()

        return response


class PNGRenderer(AbstractRenderer):
    pass



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-12 08:55
from __future__ import unicode_literals

import autoslug.fields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Blip',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('area', models.CharField(choices=[(b'techniques', 'Techniques'), (b'tools', 'Tools'), (b'platforms', 'Platforms'), (b'languages_frameworks', 'Languages & Frameworks')], max_length=32)),
                ('status', models.CharField(choices=[(b'access', 'Access'), (b'trial', 'Trial'), (b'adopt', 'Adopt'), (b'on_hold', 'On hold')], max_length=16)),
                ('name', models.CharField(max_length=255)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from=b'name')),
                ('body', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='HistoricalBlip',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('created', models.DateTimeField(blank=True, editable=False)),
                ('modified', models.DateTimeField(blank=True, editable=False)),
                ('area', models.CharField(choices=[(b'techniques', 'Techniques'), (b'tools', 'Tools'), (b'platforms', 'Platforms'), (b'languages_frameworks', 'Languages & Frameworks')], max_length=32)),
                ('status', models.CharField(choices=[(b'access', 'Access'), (b'trial', 'Trial'), (b'adopt', 'Adopt'), (b'on_hold', 'On hold')], max_length=16)),
                ('name', models.CharField(max_length=255)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from=b'name')),
                ('body', models.TextField()),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
                'verbose_name': 'historical blip',
            },
        ),
        migrations.CreateModel(
            name='Radar',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=64)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from=b'name')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='historicalblip',
            name='radar',
            field=models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='technology_radar.Radar'),
        ),
        migrations.AddField(
            model_name='blip',
            name='radar',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='blips', to='technology_radar.Radar'),
        ),
    ]



from django.shortcuts import get_object_or_404

from rest_framework.response import Response
from rest_framework.views import APIView

from technology_radar.models import Area, Status, Blip, Radar
from technology_radar.serializers import (
    AreaSerializer, StatusSerializer, BlipSerializer, RadarSerializer)


__all__ = ['AreaListView', 'StatusListView', 'RadarListView',
           'RadarDetailView', 'BlipListView', 'BlipDetailView']


class AreaListView(APIView):
    def get(self, request, format=None):
        queryset = Area.objects.all()
        serializer = AreaSerializer(queryset, many=True)
        return Response(serializer.data)


class StatusListView(APIView):
    def get(self, request, format=None):
        queryset = Status.objects.all()
        serializer = StatusSerializer(queryset, many=True)
        return Response(serializer.data)


class RadarListView(APIView):
    def get(self, request, format=None):
        queryset = Radar.objects.all()
        serializer = RadarSerializer(queryset, many=True)
        return Response(serializer.data)


class RadarDetailView(APIView):
    def get(self, request, pk, format=None):
        obj = get_object_or_404(Radar, pk=pk)
        serializer = RadarSerializer(obj)
        return Response(serializer.data)


class BlipListView(APIView):
    def get(self, request, format=None):
        queryset = Blip.objects.all()
        serializer = BlipSerializer(queryset, many=True)
        return Response(serializer.data)


class BlipDetailView(APIView):
    def get(self, request, pk, format=None):
        obj = get_object_or_404(Blip, pk=pk)
        serializer = BlipSerializer(obj)
        return Response(serializer.data)



from django.conf import settings
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404
from django.template import loader

from technology_radar.models import Area, Blip, Radar
from technology_radar.utils import import_class


__all__ = ['index', 'radar_detail', 'radar_detail_download', 'area_detail',
           'blip_detail']


def index(request):
    radars = Radar.objects.all()
    template = loader.get_template('technology_radar/index.html')
    context = {
        'radars': radars
    }
    return HttpResponse(template.render(context, request))


def radar_detail(request, radar):
    radar_obj = get_object_or_404(Radar, slug=radar)
    areas = Area.objects.all()
    template = loader.get_template('technology_radar/radar.html')
    context = {
        'radar': radar_obj,
        'areas': areas
    }
    return HttpResponse(template.render(context, request))


def radar_detail_download(request, radar):
    radar_obj = get_object_or_404(Radar, slug=radar)
    renderer = import_class(getattr(settings, 'TECHNOLOGY_RADAR_RENDER_CLASS',
                            'technology_radar.renderers.PDFRenderer'))()
    response = renderer.render(request, radar_obj)
    return response


def area_detail(request, radar, area):
    radar_obj = get_object_or_404(Radar, slug=radar)
    area_obj = get_object_or_404(Area, slug=area)
    blips = Blip.objects.filter(radar=radar_obj, area=area_obj)
    template = loader.get_template('technology_radar/area.html')
    context = {
        'radar': radar_obj,
        'area': area_obj,
        'blips': blips
    }
    return HttpResponse(template.render(context, request))


def blip_detail(request, radar, area, blip):
    blip_obj = get_object_or_404(Blip, slug=blip)
    area_obj = get_object_or_404(Area, slug=area)  # noqa
    if blip_obj.radar.slug != radar:
        raise Http404
    radar_obj = get_object_or_404(Radar, slug=radar)
    template = loader.get_template('technology_radar/blip.html')
    context = {
        'radar': radar_obj,
        'blip': blip_obj
    }
    return HttpResponse(template.render(context, request))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-12 16:03
from __future__ import unicode_literals

import autoslug.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('technology_radar', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Area',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from=b'name')),
            ],
        ),
        migrations.CreateModel(
            name='Status',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from=b'name')),
            ],
        ),
        migrations.AlterField(
            model_name='blip',
            name='area',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='blips', to='technology_radar.Area'),
        ),
        migrations.AlterField(
            model_name='blip',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='blips', to='technology_radar.Status'),
        ),
        migrations.AlterField(
            model_name='historicalblip',
            name='area',
            field=models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='technology_radar.Area'),
        ),
        migrations.AlterField(
            model_name='historicalblip',
            name='status',
            field=models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='technology_radar.Status'),
        ),
    ]



from django.conf.urls import url

from technology_radar import views


urlpatterns = [
    url(r'^api/$', views.RootView.as_view(), name='api-root'),
    url(r'^api/areas/$', views.AreaListView.as_view(), name='api-area-list'),
    url(r'^api/statuses/$', views.StatusListView.as_view(),
        name='api-status-list'),
    url(r'^api/radars/$', views.RadarListView.as_view(),
        name='api-radar-list'),
    url(r'^api/radars/(?P<pk>[0-9]+)/$', views.RadarDetailView.as_view(),
        name='api-radar-detail'),
    url(r'^api/blips/$', views.BlipListView.as_view(),
        name='api-blip-list'),
    url(r'^api/blips/(?P<pk>[0-9]+)/$', views.BlipDetailView.as_view(),
        name='api-blip-detail'),
    url(r'^$', views.index, name='radar-index'),
    url(r'^(?P<radar>[\w-]+)/$', views.radar_detail,
        name='radar-detail'),
    url(r'^(?P<radar>[\w-]+)/download/$', views.radar_detail_download,
        name='radar-detail-download'),
    url(r'^(?P<radar>[\w-]+)/(?P<area>[\w-]+)/$', views.area_detail,
        name='area-detail'),
    url(r'^(?P<radar>[\w-]+)/(?P<area>[\w-]+)/(?P<blip>[\w-]+)/$',
        views.blip_detail, name='blip-detail'),
]



from django import template


register = template.Library()


@register.assignment_tag
def get_radar_blips_by_area(radar, area):
    return radar.get_blips_by_area(area)


@register.assignment_tag
def get_radar_blips_by_status(radar, status):
    return radar.get_blips_by_status(status)



from autoslug import AutoSlugField

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _

from simple_history.models import HistoricalRecords


class TimeStampedModel(models.Model):
    created = models.DateTimeField(auto_now_add=True, editable=False)
    modified = models.DateTimeField(auto_now=True, editable=False)

    class Meta:
        abstract = True


@python_2_unicode_compatible
class Area(models.Model):
    name = models.CharField(max_length=64)
    slug = AutoSlugField(populate_from='name')

    class Meta:
        verbose_name = _('area')
        verbose_name_plural = _('area\'s')

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Status(models.Model):
    name = models.CharField(max_length=64)
    slug = AutoSlugField(populate_from='name')

    class Meta:
        verbose_name = _('status')
        verbose_name_plural = _('statuses')

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Radar(TimeStampedModel):
    name = models.CharField(max_length=64)
    slug = AutoSlugField(populate_from='name')

    class Meta:
        verbose_name = _('radar')
        verbose_name_plural = _('radars')

    def __str__(self):
        return self.name

    def get_blips_by_area(self, area):
        return Blip.objects.by_area(area.slug).filter(radar=self)

    def get_blips_by_status(self, status):
        return Blip.objects.by_status(status.slug).filter(radar=self)


class BlipManager(models.Manager):
    def by_area(self, area):
        return self.filter(area__slug=area)

    def by_status(self, status):
        return self.filter(status__slug=status)


@python_2_unicode_compatible
class Blip(TimeStampedModel):
    area = models.ForeignKey('Area', related_name='blips')
    radar = models.ForeignKey('Radar', related_name='blips')
    status = models.ForeignKey('Status', related_name='blips')
    name = models.CharField(max_length=255)
    slug = AutoSlugField(populate_from='name')
    body = models.TextField()
    history = HistoricalRecords()
    objects = BlipManager()

    class Meta:
        verbose_name = _('blip')
        verbose_name_plural = _('blips')

    def __str__(self):
        return self.name

    @property
    def is_new(self):
        return self.history.count() == 0



import pytest

from django.core.urlresolvers import reverse

from technology_radar.models import Blip
from technology_radar.tests.factories import (
    AreaFactory, StatusFactory, RadarFactory, BlipFactory)


@pytest.mark.django_db
def test_area():
    radar = AreaFactory()
    assert str(radar) == 'Tools'


@pytest.mark.django_db
def test_status():
    radar = StatusFactory()
    assert str(radar) == 'Access'


@pytest.mark.django_db
def test_radar():
    radar = RadarFactory()
    assert str(radar) == 'Moor Interactive'


@pytest.mark.django_db
def test_blip():
    blip = BlipFactory()
    assert str(blip) == 'BEM'


@pytest.mark.django_db
def test_blip_manager_by_area():
    blip = BlipFactory()
    assert Blip.objects.by_area('tools').count() == 1
    assert Blip.objects.by_area('toolz').count() == 0


@pytest.mark.django_db
def test_blip_manager_by_status():
    blip = BlipFactory()
    assert Blip.objects.by_status('access').count() == 1
    assert Blip.objects.by_status('accezz').count() == 0


@pytest.mark.django_db
def test_api_area_list(client):
    area = AreaFactory()
    res = client.get(reverse('api-area-list'))
    assert res.status_code == 200


@pytest.mark.django_db
def test_api_status_list(client):
    status = StatusFactory()
    res = client.get(reverse('api-status-list'))
    assert res.status_code == 200


@pytest.mark.django_db
def test_api_radar_list(client):
    res = client.get(reverse('api-radar-list'))
    assert res.status_code == 200


@pytest.mark.django_db
def test_api_radar_detail(client):
    radar = RadarFactory()
    res = client.get(reverse('api-radar-detail', kwargs={
        'pk': 1
    }))
    assert res.status_code == 200
    res = client.get(reverse('api-radar-detail', kwargs={
        'pk': 2
    }))
    assert res.status_code == 404


@pytest.mark.django_db
def test_api_blip_list(client):
    res = client.get(reverse('api-blip-list'))
    assert res.status_code == 200


@pytest.mark.django_db
def test_api_blip_detail(client):
    blip = BlipFactory()
    res = client.get(reverse('api-blip-detail', kwargs={
        'pk': 1
    }))
    assert res.status_code == 200
    res = client.get(reverse('api-blip-detail', kwargs={
        'pk': 2
    }))
    assert res.status_code == 404


@pytest.mark.django_db
def test_index(client):
    res = client.get(reverse('radar-index'))
    assert res.status_code == 200


@pytest.mark.django_db
def test_radar_detail(client):
    radar = RadarFactory()
    res = client.get(reverse('radar-detail', kwargs={
        'radar': 'moor-interactive'
    }))
    assert res.status_code == 200
    res = client.get(reverse('radar-detail', kwargs={
        'radar': 'moor-interactiv'
    }))
    assert res.status_code == 404


@pytest.mark.django_db
def test_radar_download(client, settings):
    radar = RadarFactory()
    res = client.get(reverse('radar-detail-download', kwargs={
        'radar': 'moor-interactive'
    }))
    assert res.status_code == 200
    res = client.get(reverse('radar-detail', kwargs={
        'radar': 'moor-interactiv'
    }))
    assert res.status_code == 404
    settings.TECHNOLOGY_RADAR_RENDER_CLASS = 'technology_radar.renderers.AbstractRenderer'
    res = client.get(reverse('radar-detail-download', kwargs={
        'radar': 'moor-interactive'
    }))
    assert res.status_code == 200


@pytest.mark.django_db
def test_area_detail(client):
    blip = BlipFactory()
    res = client.get(reverse('area-detail', kwargs={
        'radar': 'moor-interactive',
        'area': 'tools'
    }))
    assert res.status_code == 200
    res = client.get(reverse('area-detail', kwargs={
        'radar': 'moor-interactiv',
        'area': 'tools'
    }))
    assert res.status_code == 404
    res = client.get(reverse('area-detail', kwargs={
        'radar': 'moor-interactive',
        'area': 'tool'
    }))
    assert res.status_code == 404


@pytest.mark.django_db
def test_blip_detail(client):
    blip = BlipFactory()
    res = client.get(reverse('blip-detail', kwargs={
        'radar': 'moor-interactive',
        'area': 'tools',
        'blip': 'bem'
    }))
    assert res.status_code == 200
    res = client.get(reverse('blip-detail', kwargs={
        'radar': 'moor-interactive',
        'area': 'tools',
        'blip': 'be'
    }))
    assert res.status_code == 404
    radar = RadarFactory(name='Company')
    res = client.get(reverse('blip-detail', kwargs={
        'radar': 'company',
        'area': 'tools',
        'blip': 'bem'
    }))
    assert res.status_code == 404



from setuptools import find_packages, setup


install_requires = [
    'django>=1.8,<1.10',
    'django-simple-history>=1.8,<1.9',
    'django-autoslug>=1.9,<2.0',
    'djangorestframework>=3.3,<3.4',
    'reportlab>=3.3,<3.4'
]

docs_require = [
    'sphinx'
]

test_require = [
    'flake8',
    'pytest',
    'pytest-cov',
    'pytest-django',
    'pytest-factoryboy',
    'tox'
]

setup(
    name='django-technology-radar',
    version='0.1.0',
    description='A Django app for building your own Technology Radar.',
    author='Rob Moorman',
    author_email='rob@moori.nl',
    url='https://github.com/moorinteractive/django-technology-radar',
    download_url='https://github.com/moorinteractive/django-technology-radar/tarball/0.1.0',
    install_requires=install_requires,
    extras_require={
        'docs': docs_require,
        'test': test_require
    },
    packages=find_packages(exclude=[
        '*tests',
        '*fixtures',
        'sandbox'
    ]),
    include_package_data=True,
    classifier=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Operating System :: Unix',
        'Programming Language :: Python',
    ],
)



from rest_framework import serializers

from technology_radar.models import Area, Status, Blip, Radar


class AreaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Area
        fields = ('id', 'name', 'slug')


class StatusSerializer(serializers.ModelSerializer):
    class Meta:
        model = Status
        fields = ('id', 'name', 'slug')


class BlipSerializer(serializers.ModelSerializer):
    class Meta:
        model = Blip
        fields = ('id', 'area', 'status', 'name', 'slug', 'body', 'created',
                  'modified')


class RadarSerializer(serializers.ModelSerializer):
    blips = BlipSerializer(many=True)

    class Meta:
        model = Radar
        fields = ('id', 'name', 'slug', 'created', 'modified', 'blips')
